module AccountsManagement
  class Documents::Invoice < ActiveRecord::Base
    belongs_to :owner, polymorphic: true
    belongs_to :owner_account, class_name: 'AccountsManagement::Ledgers::Account'
    belongs_to :invoice_status # , class_name: 'AccountsManagement::Documents::InvoiceStatus'
    belongs_to :invoice_style # , class_name: 'AccountsManagement::Documents::InvoiceStyle'
    belongs_to :vat_rate, class_name: 'CoreModels::Misc::VatRate'
    belongs_to :currency, class_name: 'CoreModels::Base::Currency'
    belongs_to :alternate_shipping_address, class_name: 'CoreModels::Base::Address', optional: true
    belongs_to :client_account, class_name: 'AccountsManagement::Ledgers::Account'
    belongs_to :payment_method, class_name: 'CoreModels::Base::PaymentMethod', optional: true
    belongs_to :salesperson, class_name: 'CoreModels::PeopleAndCompanies::Person', optional: true

    # has_many :invoice_entries, class_name: 'AccountsManagement::Documents::InvoiceEntry', dependent: :destroy
    has_many :invoice_entries, dependent: :destroy
    accepts_nested_attributes_for :invoice_entries, reject_if: ->(a) { a[:quantity].blank? }, allow_destroy: true
    # attr_accessible :invoice_entries, :invoice_entries_attributes

    has_many :generated_invoices, class_name: 'AssetManagement::AssetData', as: :asset_owner, dependent: :destroy
    accepts_nested_attributes_for :generated_invoices, allow_destroy: true

    #---------------------------------------------------------------------------

    # attr_accessible :reference_number
    # attr_accessible :customer_purchase_order_reference
    # attr_accessible :date_prepared, :date_printed, :date_shipped
    # attr_accessible :gross_amount, :net_amount, :payment_to_date
    # attr_accessible :shipped_via, :vat_charged
    # attr_accessible :additional_notes

    # attr_accessible :invoice_style, :invoice_style_id
    # attr_accessible :invoice_status, :invoice_status_id
    # attr_accessible :vat_rate, :vat_rate_id
    # attr_accessible :currency, :currency_id
    # attr_accessible :payment_method, :payment_method_id
    # attr_accessible :salesperson, :salesperson_id
    # attr_accessible :alternate_shipping_address, :alternate_shipping_address_id
    # attr_accessible :client_account, :client_account_id

    #---------------------------------------------------------------------------

    validates :owner_account, presence: true
    validates :reference_number, presence: true
    validates :client_account, presence: true
    validates :currency, presence: true

    #---------- Callbacks

    after_initialize :after_initialize_handler
    # after_save        :after_save_handler
    # before_destroy    :before_destroy_handler

    #---------------------------------------------------------------------------

    def to_s
      reference_number
    end

    #---------- Helpers

    def self.generate_reference_nmber
      number = 1
      # Time.now.strftime("%y%m%j%H%M%S") #"YY|MM|doy|hh|mm|ss"
      # Time.now.strftime("%Y%m%d").to_s + (sprintf '%03d', number) #"YYYY|MM|DD|##"
      reference_number = Time.now.strftime("%Y%m%d") + (format '%03d', number) # "YYYY|MM|DD|##"

      # Ensure reference number is unique
      while AccountsManagement::Documents::Invoice.find_by_reference_number(reference_number)
        number += 1
        reference_number = Time.now.strftime("%Y%m%d") + (format '%03d', number) # "YYYY|MM|DD|##"
      end

      reference_number
    end

    def initialise_from_contract(contract_reference)
      contract = AccountsManagement::Documents::Contract.find_by(name: contract_reference)
      return if contract.nil?

      self.owner = contract.first_party
      self.owner_account = contract.first_party_account

      self.currency = contract.currency

      self.client_account = contract.second_party.accounts.first if contract.second_party.accounts.count >= 1

      net_amount = 0
      contract.contract_line_items.each do |contract_line_item|
        invoice_entry = AccountsManagement::Documents::InvoiceEntry.new
        invoice_entry.quantity = contract_line_item.quantity
        invoice_entry.product = contract_line_item.product
        invoice_entry.description = contract_line_item.description
        invoice_entry.unit_price = contract_line_item.product.unit_price
        invoice_entry.net_price = invoice_entry.quantity * invoice_entry.unit_price
        invoice_entry.currency = contract_line_item.currency
        invoice_entries << invoice_entry
        net_amount += invoice_entry.net_price
      end

      self.net_amount = net_amount
      vat = vat_rate.nil? ? 0.0 : vat_rate.standard_rate
      self.vat_charged = net_amount * vat / 100.0
      self.gross_amount = self.net_amount + vat_charged
    end

    #---------- Callback handlers

    private

    def after_initialize_handler
      return unless new_record?

      self.reference_number = AccountsManagement::Documents::Invoice.generate_reference_nmber

      self.date_prepared = Date.current
      self.invoice_status = AccountsManagement::Documents::InvoiceStatus.find_by_name('Draft')
      self.invoice_style = AccountsManagement::Documents::InvoiceStyle.find_by_name('Standard')
      self.vat_rate = CoreModels::Misc::VatRate.current_rate
      self.additional_notes = 'This is a test' if Rails.env.development?
    end

    def after_save_handler
    end

    def before_destroy_handler
    end
  end
end
